@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,60 +10,26 @@ import sys
 import tempfile
 from abc import abstractmethod
 from argparse import ArgumentParser, Namespace
-from typing import Any, List, NamedTuple, Optional, Tuple
+from typing import Tuple, List
 
 import aiofiles
 from idb.cli import ClientCommand
-from idb.common.types import Client, FileContainer, FileContainerType
+from idb.common.signal import signal_handler_event
+from idb.common.types import Client, FileContainer, FileContainerType, Compression
 
 
-class BundleWithPath(NamedTuple):
-    bundle_id: Optional[str]
-    path: str
-
-    @classmethod
-    def parse(cls, argument: str) -> "BundleWithPath":
-        split = argument.split(sep=":", maxsplit=1)
-        if len(split) == 1:
-            return BundleWithPath(bundle_id=None, path=split[0])
-        return BundleWithPath(bundle_id=split[0], path=split[1])
-
-
-def _extract_bundle_id(args: Namespace) -> FileContainer:
-    if args.bundle_id is not None:
-        return args.bundle_id
-    values = []
-    for value in vars(args).values():
-        if isinstance(value, List):
-            values.extend(value)
-        else:
-            values.append(value)
-    for value in values:
-        if not isinstance(value, BundleWithPath):
-            continue
-        bundle_id = value.bundle_id
-        if bundle_id is None:
-            continue
-        args.bundle_id = bundle_id
-    return args.bundle_id
-
-
-def _convert_args(args: Namespace) -> Tuple[Namespace, FileContainer]:
-    def convert_value(value: Any) -> Any:  # pyre-ignore
-        if isinstance(value, List):
-            return [convert_value(x) for x in value]
-        return value.path if isinstance(value, BundleWithPath) else value
-
-    bundle_id = _extract_bundle_id(args)
-    args = Namespace(
-        **{
-            key: convert_value(value)
-            for (key, value) in vars(args).items()
-            if key != "bundle_id"
-        }
-    )
-    file_container = bundle_id or args.container_type
-    return (args, file_container)
+def _add_container_types_to_group(
+    parser: ArgumentParser, containers: List[Tuple[FileContainerType, str]]
+) -> None:
+    for (container_type, help_text) in containers:
+        argument_name = container_type.value.replace("_", "-")
+        parser.add_argument(
+            f"--{argument_name}",
+            action="store_const",
+            dest="container_type",
+            const=container_type,
+            help=help_text,
+        )
 
 
 class FSCommand(ClientCommand):
@@ -71,59 +37,55 @@ class FSCommand(ClientCommand):
         group = parser.add_mutually_exclusive_group()
         group.add_argument(
             "--bundle-id",
-            help="Bundle ID of application. If not provided, the 'root' of the target will be used",
+            help="DEPRECATED: Use --application instead. Bundle ID of application. If not provided, the 'root' of the target will be used.",
             type=str,
             required=False,
             default=None,
         )
-        group.add_argument(
-            "--root",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.ROOT,
-            help="Use the root file container",
-        )
-        group.add_argument(
-            "--media",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.MEDIA,
-            help="Use the media container",
-        )
-        group.add_argument(
-            "--crashes",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.CRASHES,
-            help="Use the crashes container",
-        )
-        group.add_argument(
-            "--provisioning-profiles",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.PROVISIONING_PROFILES,
-            help="Use the provisioning profiles container",
-        )
-        group.add_argument(
-            "--mdm-profiles",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.MDM_PROFILES,
-            help="Use the mdm profiles container",
-        )
-        group.add_argument(
-            "--springboard-icons",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.SPRINGBOARD_ICONS,
-            help="Use the springboard icons container",
-        )
-        group.add_argument(
-            "--wallpaper",
-            action="store_const",
-            dest="container_type",
-            const=FileContainerType.WALLPAPER,
-            help="Use the wallpaper container",
+        _add_container_types_to_group(
+            group,  # pyre-fixme[6]: _MutuallyExclusiveGroup is not public.
+            [
+                (
+                    FileContainerType.APPLICATION,
+                    "Use the container of application containers. Applications containers are presented, by bundle-id in the root.",
+                ),
+                (
+                    FileContainerType.AUXILLARY,
+                    "Use the auxillary container. This is where idb will store intermediate files.",
+                ),
+                (
+                    FileContainerType.GROUP,
+                    "Use the group containers. Group containers are shared directories between applications and are prefixed with reverse-domain identifiers (e.g 'group.com.apple.safari')",
+                ),
+                (FileContainerType.ROOT, "Use the root file container"),
+                (FileContainerType.MEDIA, "Use the media container"),
+                (FileContainerType.CRASHES, "Use the crashes container"),
+                (FileContainerType.DISK_IMAGES, "Use the disk images"),
+                (
+                    FileContainerType.PROVISIONING_PROFILES,
+                    "Use the provisioning profiles container",
+                ),
+                (FileContainerType.MDM_PROFILES, "Use the mdm profiles container"),
+                (
+                    FileContainerType.SPRINGBOARD_ICONS,
+                    "Use the springboard icons container",
+                ),
+                (FileContainerType.WALLPAPER, "Use the wallpaper container"),
+                (
+                    FileContainerType.XCTEST,
+                    "Use the container of installed xctest bundles",
+                ),
+                (FileContainerType.DYLIB, "Use the container of installed dylibs"),
+                (FileContainerType.DSYM, "Use the container of installed dsyms"),
+                (
+                    FileContainerType.FRAMEWORK,
+                    "Use the container of installed frameworks",
+                ),
+                (
+                    FileContainerType.SYMBOLS,
+                    "Use the container of target-provided symbols/dyld cache",
+                ),
+            ],
         )
         super().add_parser_arguments(parser)
 
@@ -134,7 +96,14 @@ class FSCommand(ClientCommand):
         pass
 
     async def run_with_client(self, args: Namespace, client: Client) -> None:
-        (args, container) = _convert_args(args)
+        bundle_id = args.bundle_id
+        if bundle_id is not None:
+            container = bundle_id
+            self.logger.warn(
+                f"'--bundle-id {bundle_id}' is deprecated, please use --application prefixing '{bundle_id}' in the file path/s provided to this command."
+            )
+        else:
+            container = args.container_type
         return await self.run_with_container(
             container=container, args=args, client=client
         )
@@ -159,7 +128,7 @@ class FSListCommand(FSCommand):
             help="Source path",
             nargs="+",
             default="./",
-            type=BundleWithPath.parse,
+            type=str,
         )
         parser.add_argument(
             "--force-new-output",
@@ -208,7 +177,9 @@ class FSMkdirCommand(FSCommand):
     def add_parser_arguments(self, parser: ArgumentParser) -> None:
         super().add_parser_arguments(parser)
         parser.add_argument(
-            "path", help="Path to directory to create", type=BundleWithPath.parse
+            "path",
+            help="Path to directory to create",
+            type=str,
         )
 
     async def run_with_container(
@@ -235,12 +206,12 @@ class FSMoveCommand(FSCommand):
             "src",
             help="Source paths relative to Container",
             nargs="+",
-            type=BundleWithPath.parse,
+            type=str,
         )
         parser.add_argument(
             "dst",
             help="Destination path relative to Container",
-            type=BundleWithPath.parse,
+            type=str,
         )
         super().add_parser_arguments(parser)
 
@@ -268,7 +239,7 @@ class FSRemoveCommand(FSCommand):
             "path",
             help="Path of item to remove (A directory will be recursively deleted)",
             nargs="+",
-            type=BundleWithPath.parse,
+            type=str,
         )
         super().add_parser_arguments(parser)
 
@@ -297,17 +268,21 @@ class FSPushCommand(FSCommand):
                 "Directory relative to the data container of the application\n"
                 "to copy the files into. Will be created if non-existent"
             ),
-            type=BundleWithPath.parse,
+            type=str,
         )
         super().add_parser_arguments(parser)
 
     async def run_with_container(
         self, container: FileContainer, args: Namespace, client: Client
     ) -> None:
+        compression = (
+            Compression[args.compression] if args.compression is not None else None
+        )
         return await client.push(
             container=container,
             src_paths=[os.path.abspath(path) for path in args.src_paths],
             dest_path=args.dest_path,
+            compression=compression,
         )
 
 
@@ -322,7 +297,9 @@ class FSPullCommand(FSCommand):
 
     def add_parser_arguments(self, parser: ArgumentParser) -> None:
         parser.add_argument(
-            "src", help="Relative Container source path", type=BundleWithPath.parse
+            "src",
+            help="Relative Container source path",
+            type=str,
         )
         parser.add_argument("dst", help="Local destination path", type=str)
         super().add_parser_arguments(parser)
@@ -350,7 +327,9 @@ class FBSReadCommand(FSCommand):
 
     def add_parser_arguments(self, parser: ArgumentParser) -> None:
         parser.add_argument(
-            "src", help="Relatve Container source path", type=BundleWithPath.parse
+            "src",
+            help="Relatve Container source path",
+            type=str,
         )
         super().add_parser_arguments(parser)
 
@@ -389,6 +368,9 @@ class FSWriteCommand(FSCommand):
     ) -> None:
         data = sys.stdin.buffer.read()
         (destination_directory, destination_file_path) = os.path.split(args.dst)
+        compression = (
+            Compression[args.compression] if args.compression is not None else None
+        )
 
         with tempfile.TemporaryDirectory() as temporary_directory:
             temporary_file_path = os.path.join(
@@ -400,4 +382,28 @@ class FSWriteCommand(FSCommand):
                 src_paths=[temporary_file_path],
                 container=container,
                 dest_path=destination_directory,
+                compression=compression,
             )
+
+
+class FSTailCommand(FSCommand):
+    @property
+    def description(self) -> str:
+        return "Tails a remote file to stdout"
+
+    @property
+    def name(self) -> str:
+        return "tail"
+
+    def add_parser_arguments(self, parser: ArgumentParser) -> None:
+        parser.add_argument("src", help="Relatve container source path", type=str)
+        super().add_parser_arguments(parser)
+
+    async def run_with_container(
+        self, container: FileContainer, args: Namespace, client: Client
+    ) -> None:
+        async for data in client.tail(
+            container=container, path=args.src, stop=signal_handler_event("tail")
+        ):
+            sys.stdout.buffer.write(data)
+            sys.stdout.flush()
