@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -65,7 +65,7 @@ class TestParser(TestCase):
             return_value=self.client_mock
         )
         self.client_manager_patch = patch(
-            "idb.cli.ClientManager", self.client_manager_mock
+            "idb.cli.GrpcClientManager", self.client_manager_mock
         )
         self.client_manager_patch.start()
         self.client_patch = patch("idb.cli.GrpcClient", self.client_mock)
@@ -91,6 +91,25 @@ class TestParser(TestCase):
             foreground_if_running=False,
             wait_for_debugger=False,
             stop=None,
+            pid_file=None,
+        )
+
+    async def test_launch_with_pid_file(self) -> None:
+        bundle_id = "com.foo.app"
+        udid = "my udid"
+        pid_file = "/tmp/fifo-file"
+        self.client_mock.launch = AsyncMock(return_value=bundle_id)
+        await cli_main(
+            cmd_input=["launch", "--pid-file", pid_file, "--udid", udid, bundle_id]
+        )
+        self.client_mock.launch.assert_called_once_with(
+            bundle_id=bundle_id,
+            env={},
+            args=[],
+            foreground_if_running=False,
+            wait_for_debugger=False,
+            stop=None,
+            pid_file=pid_file,
         )
 
     async def test_create(self) -> None:
@@ -150,7 +169,9 @@ class TestParser(TestCase):
         app_path = "testApp.app"
         compression = None
         await cli_main(cmd_input=["install", app_path])
-        self.client_mock.install.assert_called_once_with(app_path, compression)
+        self.client_mock.install.assert_called_once_with(
+            bundle=app_path, make_debuggable=None, compression=compression
+        )
 
     async def test_install_with_bad_compression(self) -> None:
         self.client_mock.install = MagicMock(return_value=AsyncGeneratorMock())
@@ -165,7 +186,9 @@ class TestParser(TestCase):
         self.client_mock.install = MagicMock(return_value=AsyncGeneratorMock())
         app_path = "testApp.app"
         await cli_main(cmd_input=["--compression", "ZSTD", "install", app_path])
-        self.client_mock.install.assert_called_once_with(app_path, Compression.ZSTD)
+        self.client_mock.install.assert_called_once_with(
+            bundle=app_path, make_debuggable=None, compression=Compression.ZSTD
+        )
 
     async def test_uninstall(self) -> None:
         self.client_mock.uninstall = AsyncMock()
@@ -186,7 +209,6 @@ class TestParser(TestCase):
         self.client_mock.list_apps.assert_called_once()
         self.client_mock.build.assert_called_once_with(
             address=TCPAddress(host="thehost", port=123),
-            is_local=False,
             logger=ANY,
             use_tls=True,
         )
@@ -197,7 +219,6 @@ class TestParser(TestCase):
         self.client_mock.list_apps.assert_called_once()
         self.client_mock.build.assert_called_once_with(
             address=DomainSocketAddress(path="/foo/sock"),
-            is_local=False,
             logger=ANY,
             use_tls=False,
         )
@@ -241,7 +262,7 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.mkdir.assert_called_once_with(container=None, path=src)
 
-    async def test_bundled_file_mkdir_flag(self) -> None:
+    async def test_bundled_file_mkdir(self) -> None:
         self.client_mock.mkdir = AsyncMock()
         src = "path"
         bundle_id = "com.bundle.id"
@@ -249,14 +270,6 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.mkdir.assert_called_once_with(container=bundle_id, path=src)
 
-    async def test_bundled_file_mkdir_colon(self) -> None:
-        self.client_mock.mkdir = AsyncMock()
-        cmd_input = ["file", "mkdir", "com.bundle.id:path"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.mkdir.assert_called_once_with(
-            container="com.bundle.id", path="path"
-        )
-
     async def test_rm(self) -> None:
         self.client_mock.rm = AsyncMock()
         src = "path"
@@ -264,7 +277,7 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.rm.assert_called_once_with(container=None, paths=[src])
 
-    async def test_bundled_file_rmpath_flag(self) -> None:
+    async def test_bundled_file_rmpath(self) -> None:
         self.client_mock.rm = AsyncMock()
         src = "path"
         bundle_id = "com.bundle.id"
@@ -272,7 +285,7 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.rm.assert_called_once_with(container=bundle_id, paths=[src])
 
-    async def test_bundled_file_rmpath_flag_multiple(self) -> None:
+    async def test_bundled_file_rmpath_multiple(self) -> None:
         self.client_mock.rm = AsyncMock()
         srcs = ["pathA", "pathB"]
         bundle_id = "com.bundle.id"
@@ -280,14 +293,6 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.rm.assert_called_once_with(container=bundle_id, paths=srcs)
 
-    async def test_bundled_file_rmpath_colon(self) -> None:
-        self.client_mock.rm = AsyncMock()
-        cmd_input = ["file", "remove", "com.bundle.id:path"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.rm.assert_called_once_with(
-            container="com.bundle.id", paths=["path"]
-        )
-
     async def test_list(self) -> None:
         self.client_mock.ls_single = AsyncMock(return_value=[])
         src = "path"
@@ -295,7 +300,7 @@ class TestParser(TestCase):
         await cli_main(cmd_input=cmd_input)
         self.client_mock.ls_single.assert_called_once_with(container=None, path=src)
 
-    async def test_bundled_file_list_flag(self) -> None:
+    async def test_bundled_file_list(self) -> None:
         self.client_mock.ls_single = AsyncMock(return_value=[])
         src = "path"
         bundle_id = "com.bundle.id"
@@ -305,14 +310,6 @@ class TestParser(TestCase):
             container=bundle_id, path=src
         )
 
-    async def test_bundled_file_list_colon(self) -> None:
-        self.client_mock.ls_single = AsyncMock(return_value=[])
-        cmd_input = ["file", "list", "com.bundle.id:path"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.ls_single.assert_called_once_with(
-            container="com.bundle.id", path="path"
-        )
-
     async def test_file_move(self) -> None:
         self.client_mock.mv = AsyncMock(return_value=[])
         src = "a"
@@ -323,7 +320,7 @@ class TestParser(TestCase):
             container=None, src_paths=[src], dest_path=dst
         )
 
-    async def test_bundled_file_move_flag(self) -> None:
+    async def test_bundled_file_move(self) -> None:
         self.client_mock.mv = AsyncMock(return_value=[])
         src = "a"
         dst = "b"
@@ -334,7 +331,7 @@ class TestParser(TestCase):
             container=bundle_id, src_paths=[src], dest_path=dst
         )
 
-    async def test_bundled_file_move_flag_multiple(self) -> None:
+    async def test_bundled_file_move_multiple(self) -> None:
         self.client_mock.mv = AsyncMock(return_value=[])
         srcs = ["src1", "src2"]
         dst = "b"
@@ -345,15 +342,6 @@ class TestParser(TestCase):
             container=bundle_id, src_paths=srcs, dest_path=dst
         )
 
-    async def test_bundled_file_move_colon(self) -> None:
-        self.client_mock.mv = AsyncMock(return_value=[])
-        src = "a"
-        cmd_input = ["file", "move", src, "com.bundle.id:b"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.mv.assert_called_once_with(
-            container="com.bundle.id", src_paths=[src], dest_path="b"
-        )
-
     async def test_file_push(self) -> None:
         self.client_mock.push = AsyncMock()
         src = "Library/myFile.txt"
@@ -361,10 +349,13 @@ class TestParser(TestCase):
         cmd_input = ["file", "push", src, dst]
         await cli_main(cmd_input=cmd_input)
         self.client_mock.push.assert_called_once_with(
-            container=None, src_paths=[os.path.abspath(src)], dest_path=dst
+            container=None,
+            src_paths=[os.path.abspath(src)],
+            dest_path=dst,
+            compression=None,
         )
 
-    async def test_bundled_file_push_single_flag(self) -> None:
+    async def test_bundled_file_push_single(self) -> None:
         self.client_mock.push = AsyncMock()
         bundle_id = "com.myapp"
         src = "Library/myFile.txt"
@@ -372,10 +363,36 @@ class TestParser(TestCase):
         cmd_input = ["file", "push", src, dst, "--bundle-id", bundle_id]
         await cli_main(cmd_input=cmd_input)
         self.client_mock.push.assert_called_once_with(
-            container=bundle_id, src_paths=[os.path.abspath(src)], dest_path=dst
+            container=bundle_id,
+            src_paths=[os.path.abspath(src)],
+            dest_path=dst,
+            compression=None,
         )
 
-    async def test_bundled_file_push_multi_flag(self) -> None:
+    async def test_bundled_file_push_zstd(self) -> None:
+        self.client_mock.push = AsyncMock()
+        bundle_id = "com.myapp"
+        src = "Library/myFile.txt"
+        dst = "someOutputDir"
+        cmd_input = [
+            "--compression",
+            "ZSTD",
+            "file",
+            "push",
+            src,
+            dst,
+            "--bundle-id",
+            bundle_id,
+        ]
+        await cli_main(cmd_input=cmd_input)
+        self.client_mock.push.assert_called_once_with(
+            container=bundle_id,
+            src_paths=[os.path.abspath(src)],
+            dest_path=dst,
+            compression=Compression.ZSTD,
+        )
+
+    async def test_bundled_file_push_multi(self) -> None:
         self.client_mock.push = AsyncMock(return_value=[])
         bundle_id = "com.myapp"
         src1 = "Library/myFile.txt"
@@ -387,29 +404,7 @@ class TestParser(TestCase):
             container=bundle_id,
             src_paths=[os.path.abspath(path) for path in [src1, src2]],
             dest_path=dst,
-        )
-
-    async def test_bundled_file_push_single_colon(self) -> None:
-        self.client_mock.push = AsyncMock(return_value=[])
-        src = "Library/myFile.txt"
-        cmd_input = ["file", "push", src, "com.bundle.id:someOutputDir"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.push.assert_called_once_with(
-            container="com.bundle.id",
-            src_paths=[os.path.abspath(src)],
-            dest_path="someOutputDir",
-        )
-
-    async def test_bundled_file_push_multi_colon(self) -> None:
-        self.client_mock.push = AsyncMock(return_value=[])
-        src1 = "Library/myFile.txt"
-        src2 = "Library/myFile2.txt"
-        cmd_input = ["file", "push", src1, src2, "com.bundle.id:someOutputDir"]
-        await cli_main(cmd_input=cmd_input)
-        self.client_mock.push.assert_called_once_with(
-            container="com.bundle.id",
-            src_paths=[os.path.abspath(path) for path in [src1, src2]],
-            dest_path="someOutputDir",
+            compression=None,
         )
 
     async def test_file_pull(self) -> None:
@@ -453,7 +448,6 @@ class TestParser(TestCase):
         namespace = Namespace()
         namespace.companion_path = COMPANION_PATH
         namespace.companion = None
-        namespace.companion_local = False
         namespace.compression = None
         namespace.prune_dead_companion = True
         namespace.log_level = "WARNING"
@@ -476,6 +470,7 @@ class TestParser(TestCase):
         namespace.report_attachments = False
         namespace.activities_output_path = None
         namespace.coverage_output_path = None
+        namespace.coverage_format = "EXPORTED"
         namespace.log_directory_path = None
         namespace.wait_for_debugger = False
         namespace.install = False
@@ -559,7 +554,6 @@ class TestParser(TestCase):
             namespace = Namespace()
             namespace.companion_path = COMPANION_PATH
             namespace.companion = None
-            namespace.companion_local = False
             namespace.compression = None
             namespace.prune_dead_companion = True
             namespace.daemon_port = port
@@ -588,7 +582,6 @@ class TestParser(TestCase):
             namespace = Namespace()
             namespace.companion_path = COMPANION_PATH
             namespace.companion = None
-            namespace.companion_local = False
             namespace.compression = None
             namespace.prune_dead_companion = True
             namespace.log_level = "WARNING"
@@ -607,7 +600,6 @@ class TestParser(TestCase):
             namespace = Namespace()
             namespace.companion_path = COMPANION_PATH
             namespace.companion = None
-            namespace.companion_local = False
             namespace.compression = None
             namespace.prune_dead_companion = True
             namespace.log_level = "WARNING"
@@ -663,7 +655,6 @@ class TestParser(TestCase):
             namespace = Namespace()
             namespace.companion_path = COMPANION_PATH
             namespace.companion = None
-            namespace.companion_local = False
             namespace.compression = None
             namespace.prune_dead_companion = True
             namespace.log_level = "WARNING"
@@ -686,7 +677,6 @@ class TestParser(TestCase):
                 Namespace(
                     companion=None,
                     companion_path=COMPANION_PATH,
-                    companion_local=False,
                     compression=None,
                     prune_dead_companion=True,
                     log_level="WARNING",
