@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,8 +11,10 @@ from typing import Any, Dict, List, Optional, Union
 from uuid import uuid4
 
 from idb.common.types import (
+    IdbException,
     AppProcessState,
     CompanionInfo,
+    DebuggerInfo,
     DomainSocketAddress,
     InstalledAppInfo,
     InstalledTestInfo,
@@ -20,21 +22,38 @@ from idb.common.types import (
     TCPAddress,
     TestActivity,
     TestRunInfo,
+    TargetType,
 )
 from treelib import Tree
 
 
+def target_type_from_string(output: str) -> TargetType:
+    normalized = output.lower()
+    if "sim" in normalized:
+        return TargetType.SIMULATOR
+    if "dev" in normalized:
+        return TargetType.DEVICE
+    raise IdbException(f"Could not interpret target type from {output}")
+
+
+def test_info_to_status(test: TestRunInfo) -> str:
+    if test.passed:
+        return "passed"
+    if test.crashed:
+        return "crashed"
+    return "failed"
+
+
 def human_format_test_info(test: TestRunInfo) -> str:
     output = ""
 
     info_list = [
         f"{test.bundle_name} - {test.class_name}/{test.method_name}",
-        f"Passed: {test.passed}",
-        f"Crashed: {test.crashed}",
+        f"Status: {test_info_to_status(test)}",
         f"Duration: {test.duration}",
     ]
     failure_info = test.failure_info
-    if failure_info:
+    if failure_info is not None and len(failure_info.message):
         info_list += [
             f"Failure message: {failure_info.message}",
             f"Location {failure_info.file}:{failure_info.line}",
@@ -45,8 +64,9 @@ def human_format_test_info(test: TestRunInfo) -> str:
         log_lines = indent("\n".join(test.logs), " " * 4)
         output += "\n" + indent("Logs:\n" + log_lines, " " * 4)
 
-    if test.activityLogs:
-        output += f"\n{human_format_activities(test.activityLogs)}"
+    activities = test.activityLogs
+    if activities is not None and len(activities):
+        output += f"\n{human_format_activities(activities)}"
     return output
 
 
@@ -54,6 +74,7 @@ def human_format_activities(activities: List[TestActivity]) -> str:
     tree: Tree = Tree()
     start = activities[0].start
 
+    # pyre-fixme[53]: Captured variable `start` is not annotated.
     def process_activity(activity: TestActivity, parent: Optional[str] = None) -> None:
         tree.create_node(
             f"{activity.name} ({activity.finish - start:.2f}s)",
@@ -79,8 +100,7 @@ def human_format_activities(activities: List[TestActivity]) -> str:
 
 
 def json_format_test_info(test: TestRunInfo) -> str:
-    failure_info = test.failure_info
-    data = {
+    data: Dict[str, Any] = {
         "bundleName": test.bundle_name,
         "className": test.class_name,
         "methodName": test.method_name,
@@ -88,17 +108,20 @@ def json_format_test_info(test: TestRunInfo) -> str:
         "duration": test.duration,
         "passed": test.passed,
         "crashed": test.crashed,
-        "failureInfo": {
+        "status": test_info_to_status(test),
+    }
+    failure_info = test.failure_info
+    if failure_info is not None and len(failure_info.message):
+        data["failureInfo"] = {
             "message": failure_info.message,
             "file": failure_info.file,
             "line": failure_info.line,
         }
-        if failure_info
-        else None,
-        "activityLogs": [
-            json_format_activity(activity) for activity in (test.activityLogs or [])
-        ],
-    }
+    activities = test.activityLogs
+    if activities is not None and len(activities):
+        data["activityLogs"] = [
+            json_format_activity(activity) for activity in activities
+        ]
     return json.dumps(data)
 
 
@@ -136,10 +159,20 @@ def human_format_installed_app_info(app: InstalledAppInfo) -> str:
             ", ".join(app.architectures or ["no archs available"]),
             app_process_state_to_string(app.process_state),
             "Debuggable" if app.debuggable else "Not Debuggable",
+            f"pid={app_process_id_based_on_state(app.process_id, app.process_state)}",
         ]
     )
 
 
+def app_process_id_based_on_state(
+    pid: int,
+    state: AppProcessState,
+) -> Optional[str]:
+    if state is AppProcessState.RUNNING:
+        return str(pid)
+    return None
+
+
 def app_process_state_to_string(state: Optional[AppProcessState]) -> str:
     if state is AppProcessState.RUNNING:
         return "Running"
@@ -166,6 +199,7 @@ def json_format_installed_app_info(app: InstalledAppInfo) -> str:
         "architectures": list(app.architectures) if app.architectures else None,
         "process_state": app_process_state_to_string(app.process_state),
         "debuggable": app.debuggable,
+        "pid": app_process_id_based_on_state(app.process_id, app.process_state),
     }
     return json.dumps(data)
 
@@ -173,7 +207,7 @@ def json_format_installed_app_info(app: InstalledAppInfo) -> str:
 def human_format_target_info(target: TargetDescription) -> str:
     target_info = (
         f"{target.name} | {target.udid} | {target.state}"
-        f" | {target.target_type} | {target.os_version} | {target.architecture} | "
+        f" | {target.target_type.value} | {target.os_version} | {target.architecture} | "
     )
     companion_info = target.companion_info
     if companion_info is None:
@@ -190,7 +224,7 @@ def json_data_target_info(target: TargetDescription) -> Dict[str, Any]:
         "name": target.name,
         "udid": target.udid,
         "state": target.state,
-        "type": target.target_type,
+        "type": target.target_type.value,
         "os_version": target.os_version,
         "architecture": target.architecture,
     }
@@ -201,9 +235,11 @@ def json_data_target_info(target: TargetDescription) -> Dict[str, Any]:
             data["host"] = address.host
             data["port"] = address.port
             data["is_local"] = companion_info.is_local
+            data["companion"] = f"{address.host}:{address.port}"
         else:
             data["path"] = address.path
             data["is_local"] = True
+            data["companion"] = address.path
     if target.device is not None:
         data["device"] = target.device
     return data
@@ -211,12 +247,13 @@ def json_data_target_info(target: TargetDescription) -> Dict[str, Any]:
 
 def json_data_companions(
     companions: List[CompanionInfo],
-) -> List[Dict[str, Union[str, int]]]:
-    data: List[Dict[str, Union[str, int]]] = []
+) -> List[Dict[str, Union[str, Optional[int]]]]:
+    data: List[Dict[str, Union[str, Optional[int]]]] = []
     for companion in companions:
-        item: Dict[str, Union[str, int]] = {
+        item: Dict[str, Union[str, Optional[int]]] = {
             "udid": companion.udid,
             "is_local": companion.is_local,
+            "pid": companion.pid,
         }
         address = companion.address
         if isinstance(address, TCPAddress):
@@ -238,6 +275,7 @@ def json_to_companion_info(data: List[Dict[str, Any]]) -> List[CompanionInfo]:
                 else DomainSocketAddress(path=item["path"])
             ),
             is_local=item["is_local"],
+            pid=item.get("pid"),
         )
         for item in data
     ]
@@ -259,7 +297,7 @@ def target_description_from_dictionary(parsed: Dict[str, Any]) -> TargetDescript
         name=parsed["name"],
         model=parsed.get("model"),
         state=parsed.get("state"),
-        target_type=parsed.get("type"),
+        target_type=target_type_from_string(parsed["type"]),
         os_version=parsed.get("os_version"),
         architecture=parsed.get("architecture"),
         companion_info=None,
@@ -289,3 +327,10 @@ def json_format_installed_test_info(test: InstalledTestInfo) -> str:
         "architectures": list(test.architectures) if test.architectures else None,
     }
     return json.dumps(data)
+
+
+def json_format_debugger_info(info: DebuggerInfo) -> str:
+    data = {
+        "pid": info.pid,
+    }
+    return json.dumps(data)
