@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -60,3 +60,14 @@ async def generate_gzip(path: str) -> AsyncIterator[bytes]:
             if not data:
                 return
             yield data
+
+
+async def _stream_from_data(data: bytes) -> AsyncIterator[bytes]:
+    yield data
+
+
+async def gunzip(data: bytes, output_path: str) -> None:
+    await drain_gzip_decompress(
+        stream=_stream_from_data(data=data),
+        output_path=output_path,
+    )
