@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,6 +7,7 @@
 import asyncio
 import json
 from abc import ABC, abstractmethod, abstractproperty
+from asyncio import StreamWriter, StreamReader
 from dataclasses import asdict, dataclass, field
 from datetime import timedelta
 from enum import Enum
@@ -52,9 +53,10 @@ class Permission(Enum):
     NOTIFICATION = 5
 
 
-class TargetType(Enum):
-    DEVICE = 1
-    SIMULATOR = 2
+class TargetType(str, Enum):
+    DEVICE = "device"
+    SIMULATOR = "simulator"
+    MAC = "mac"
 
 
 @dataclass(frozen=True)
@@ -100,6 +102,7 @@ class InstalledAppInfo:
     install_type: str
     process_state: AppProcessState
     debuggable: bool
+    process_id: int
 
 
 @dataclass(frozen=True)
@@ -125,6 +128,7 @@ ConnectionDestination = Union[str, Address]
 class CompanionInfo:
     udid: str
     is_local: bool
+    pid: Optional[int]
     address: Address
     metadata: LoggingMetadata = field(default_factory=dict)
 
@@ -145,8 +149,8 @@ DeviceDetails = Mapping[str, Union[int, str]]
 class TargetDescription:
     udid: str
     name: str
+    target_type: TargetType
     state: Optional[str]
-    target_type: Optional[str]
     os_version: Optional[str]
     architecture: Optional[str]
     companion_info: Optional[CompanionInfo]
@@ -162,16 +166,6 @@ class TargetDescription:
         return json.dumps(asdict(self))
 
 
-@dataclass(frozen=True)
-class DaemonInfo:
-    host: str
-    port: int
-    targets: List[TargetDescription]
-
-
-ConnectResponse = Union[CompanionInfo, DaemonInfo]
-
-
 @dataclass(frozen=True)
 class FileEntryInfo:
     path: str
@@ -185,7 +179,7 @@ class FileListing:
 
 @dataclass(frozen=True)
 class AccessibilityInfo:
-    json: Optional[str]
+    json: str
 
 
 @dataclass(frozen=True)
@@ -265,6 +259,11 @@ class InstalledTestInfo:
     architectures: Optional[Set[str]]
 
 
+@dataclass(frozen=True)
+class DebuggerInfo:
+    pid: int
+
+
 class HIDDirection(Enum):
     DOWN = 0
     UP = 1
@@ -324,14 +323,22 @@ class InstalledArtifact:
 
 
 class FileContainerType(Enum):
-    ROOT = "root"
-    MEDIA = "media"
+    APPLICATION = "application"
+    AUXILLARY = "auxillary"
     CRASHES = "crashes"
-    PROVISIONING_PROFILES = "provisioning_profiles"
-    MDM_PROFILES = "mdm_profiles"
-    SPRINGBOARD_ICONS = "springboard_icons"
-    WALLPAPER = "wallpaper"
     DISK_IMAGES = "disk_images"
+    DSYM = "dsym"
+    DYLIB = "dylib"
+    FRAMEWORK = "framework"
+    GROUP = "group"
+    MDM_PROFILES = "mdm_profiles"
+    MEDIA = "media"
+    PROVISIONING_PROFILES = "provisioning_profiles"
+    ROOT = "root"
+    SPRINGBOARD_ICONS = "springboard_icons"
+    SYMBOLS = "symbols"
+    WALLPAPER = "wallpaper"
+    XCTEST = "xctest"
 
 
 FileContainer = Optional[Union[str, FileContainerType]]
@@ -342,6 +349,11 @@ class Compression(Enum):
     ZSTD = 1
 
 
+class CodeCoverageFormat(Enum):
+    EXPORTED = 0
+    RAW = 1
+
+
 class Companion(ABC):
     @abstractmethod
     async def create(
@@ -433,6 +445,7 @@ class Client(ABC):
         foreground_if_running: bool = False,
         wait_for_debugger: bool = False,
         stop: Optional[asyncio.Event] = None,
+        pid_file: Optional[str] = None,
     ) -> None:
         pass
 
@@ -456,6 +469,7 @@ class Client(ABC):
         report_attachments: bool = False,
         activities_output_path: Optional[str] = None,
         coverage_output_path: Optional[str] = None,
+        coverage_format: CodeCoverageFormat = CodeCoverageFormat.EXPORTED,
         log_directory_path: Optional[str] = None,
         wait_for_debugger: bool = False,
     ) -> AsyncIterator[TestRunInfo]:
@@ -466,6 +480,7 @@ class Client(ABC):
         self,
         bundle: Union[str, IO[bytes]],
         compression: Optional[Compression] = None,
+        make_debuggable: Optional[bool] = None,
     ) -> AsyncIterator[InstalledArtifact]:
         yield
 
@@ -477,7 +492,10 @@ class Client(ABC):
 
     @abstractmethod
     async def install_dsym(
-        self, dsym: Union[str, IO[bytes]]
+        self,
+        dsym: Union[str, IO[bytes]],
+        bundle_id: Optional[str],
+        compression: Optional[Compression],
     ) -> AsyncIterator[InstalledArtifact]:
         yield
 
@@ -531,10 +549,20 @@ class Client(ABC):
     async def set_locale(self, locale_identifier: str) -> None:
         pass
 
+    @abstractmethod
+    async def set_preference(
+        self, name: str, value: str, value_type: str, domain: Optional[str]
+    ) -> None:
+        pass
+
     @abstractmethod
     async def get_locale(self) -> str:
         pass
 
+    @abstractmethod
+    async def get_preference(self, name: str, domain: Optional[str]) -> str:
+        pass
+
     @abstractmethod
     async def list_locale_identifiers(self) -> List[str]:
         pass
@@ -547,6 +575,14 @@ class Client(ABC):
     async def set_location(self, latitude: float, longitude: float) -> None:
         pass
 
+    @abstractmethod
+    async def simulate_memory_warning(self) -> None:
+        pass
+
+    @abstractmethod
+    async def send_notification(self, bundle_id: str, json_payload: str) -> None:
+        pass
+
     @abstractmethod
     async def approve(
         self, bundle_id: str, permissions: Set[Permission], scheme: Optional[str] = None
@@ -671,6 +707,16 @@ class Client(ABC):
     async def focus(self) -> None:
         pass
 
+    async def dap(
+        self,
+        dap_path: str,
+        input_stream: StreamReader,
+        output_stream: StreamWriter,
+        stop: asyncio.Event,
+        compression: Optional[Compression],
+    ) -> None:
+        raise NotImplementedError("Dap command not implemented")
+
     @abstractmethod
     async def debugserver_start(self, bundle_id: str) -> List[str]:
         pass
@@ -723,10 +769,20 @@ class Client(ABC):
 
     @abstractmethod
     async def push(
-        self, src_paths: List[str], container: FileContainer, dest_path: str
+        self,
+        src_paths: List[str],
+        container: FileContainer,
+        dest_path: str,
+        compression: Optional[Compression],
     ) -> None:
         pass
 
+    @abstractmethod
+    async def tail(
+        self, stop: asyncio.Event, container: FileContainer, path: str
+    ) -> AsyncIterator[bytes]:
+        yield
+
 
 class ClientManager:
     @abstractmethod
@@ -742,7 +798,9 @@ class ClientManager:
         pass
 
     @abstractmethod
-    async def list_targets(self) -> List[TargetDescription]:
+    async def list_targets(
+        self, only: Optional[OnlyFilter] = None
+    ) -> List[TargetDescription]:
         pass
 
     @abstractmethod
