@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,8 +10,10 @@ import inspect
 import logging
 import os
 import shutil
+import sys
 import tempfile
 import urllib.parse
+from asyncio import StreamWriter, StreamReader
 from io import StringIO
 from pathlib import Path
 from typing import (
@@ -32,7 +34,8 @@ from grpclib.client import Channel
 from grpclib.exceptions import GRPCError, ProtocolError, StreamTerminatedError
 from idb.common.constants import TESTS_POLL_INTERVAL
 from idb.common.file import drain_to_file
-from idb.common.gzip import drain_gzip_decompress
+from idb.common.format import json_format_debugger_info
+from idb.common.gzip import drain_gzip_decompress, gunzip
 from idb.common.hid import (
     button_press_to_events,
     iterator_to_async_iterator,
@@ -45,10 +48,12 @@ from idb.common.logging import log_call
 from idb.common.stream import stream_map
 from idb.common.tar import create_tar, drain_untar, generate_tar
 from idb.common.types import (
+    FileContainerType,
     AccessibilityInfo,
     Address,
     AppProcessState,
     Client as ClientBase,
+    CodeCoverageFormat,
     Companion,
     CompanionInfo,
     Compression,
@@ -73,12 +78,14 @@ from idb.common.types import (
     TCPAddress,
     TestRunInfo,
     VideoFormat,
+    DebuggerInfo,
 )
 from idb.grpc.crash import (
     _to_crash_log,
     _to_crash_log_info_list,
     _to_crash_log_query_proto,
 )
+from idb.grpc.dap import RemoteDapServer
 from idb.grpc.file import container_to_grpc as file_container_to_grpc
 from idb.grpc.hid import event_to_grpc
 from idb.grpc.idb_grpc import CompanionServiceStub
@@ -96,12 +103,16 @@ from idb.grpc.idb_pb2 import (
     InstallRequest,
     InstrumentsRunRequest,
     LaunchRequest,
+    TailRequest,
     ListAppsRequest,
     Location,
     LogRequest,
+    SimulateMemoryWarningRequest,
+    SendNotificationRequest,
     LsRequest,
     GetSettingRequest,
     LOCALE as LocaleSetting,
+    ANY as AnySetting,
     ListSettingRequest,
     MkdirRequest,
     MvRequest,
@@ -122,6 +133,7 @@ from idb.grpc.idb_pb2 import (
     XctestListBundlesRequest,
     XctestListTestsRequest,
     XctraceRecordRequest,
+    XctestRunResponse,
 )
 from idb.grpc.install import (
     Bundle,
@@ -239,7 +251,6 @@ class Client(ClientBase):
         cls,
         address: Address,
         logger: logging.Logger,
-        is_local: Optional[bool] = None,
         exchange_metadata: bool = True,
         extra_metadata: Optional[Dict[str, str]] = None,
         use_tls: bool = False,
@@ -281,9 +292,10 @@ class Client(ClientBase):
                     raise IdbException(
                         f"Failed to connect to companion at address {address}: {ex}"
                     )
-            companion = companion_to_py(
-                companion=response.companion, address=address, is_local=is_local
+            logger.debug(
+                f"Companion at {address} {'is' if response.companion.is_local else 'is not'} local"
             )
+            companion = companion_to_py(companion=response.companion, address=address)
             if exchange_metadata:
                 metadata_from_companion = {
                     key: value
@@ -311,7 +323,6 @@ class Client(ClientBase):
                 udid=udid, path=temp.name, only=only
             ) as resolved_path, Client.build(
                 address=DomainSocketAddress(path=resolved_path),
-                is_local=True,
                 logger=logger,
             ) as client:
                 yield client
@@ -333,7 +344,9 @@ class Client(ClientBase):
         self,
         bundle: Bundle,
         destination: Destination,
-        compression: Optional[Compression] = None,
+        compression: Optional[Compression],
+        make_debuggable: Optional[bool],
+        bundle_id: Optional[str],
     ) -> AsyncIterator[InstalledArtifact]:
         async with self.stub.install.open() as stream:
             generator = None
@@ -347,11 +360,17 @@ class Client(ClientBase):
                 else:
                     file_path = str(Path(bundle).resolve(strict=True))
                     if self.is_local:
+                        self.logger.debug(
+                            f"Companion is local, sending local file by path {file_path}"
+                        )
                         # send file_path
                         generator = generate_requests(
                             [InstallRequest(payload=Payload(file_path=file_path))]
                         )
                     else:
+                        self.logger.debug(
+                            f"Companion is remote, generating binary chunks for {file_path}"
+                        )
                         # chunk file from file_path
                         generator = generate_binary_chunks(
                             path=file_path,
@@ -362,17 +381,25 @@ class Client(ClientBase):
 
             else:
                 # chunk file from memory
+                self.logger.debug("Sending file data from input stream")
                 generator = generate_io_chunks(io=bundle, logger=self.logger)
                 # stream to companion
             await stream.send_message(InstallRequest(destination=destination))
+            if make_debuggable is not None:
+                await stream.send_message(
+                    InstallRequest(make_debuggable=make_debuggable)
+                )
             if compression is not None:
                 await stream.send_message(
                     InstallRequest(
                         payload=Payload(compression=COMPRESSION_MAP[compression])
                     )
                 )
+            if bundle_id is not None:
+                await stream.send_message(InstallRequest(bundle_id=bundle_id))
             async for message in generator:
                 await stream.send_message(message)
+            self.logger.debug("Finished sending install payload to companion")
             await stream.end()
             async for response in stream:
                 yield InstalledArtifact(
@@ -401,6 +428,7 @@ class Client(ClientBase):
                 install_type=app.install_type,
                 process_state=AppProcessState(app.process_state),
                 debuggable=app.debuggable,
+                process_id=app.process_identifier,
             )
             for app in response.apps
         ]
@@ -482,6 +510,19 @@ class Client(ClientBase):
             )
         )
 
+    @log_and_handle_exceptions
+    async def simulate_memory_warning(self) -> None:
+        await self.stub.simulate_memory_warning(SimulateMemoryWarningRequest())
+
+    @log_and_handle_exceptions
+    async def send_notification(self, bundle_id: str, json_payload: str) -> None:
+        await self.stub.send_notification(
+            SendNotificationRequest(
+                bundle_id=bundle_id,
+                json_payload=json_payload,
+            )
+        )
+
     @log_and_handle_exceptions
     async def terminate(self, bundle_id: str) -> None:
         await self.stub.terminate(TerminateRequest(bundle_id=bundle_id))
@@ -496,7 +537,10 @@ class Client(ClientBase):
             target=target,
             # Use the local understanding of the companion instead of the remote's.
             companion=CompanionInfo(
-                address=self.address, udid=target.udid, is_local=self.is_local
+                address=self.address,
+                udid=target.udid,
+                is_local=self.is_local,
+                pid=None,
             ),
             # Extract the companion metadata from the response.
             metadata=response.companion.metadata,
@@ -580,30 +624,52 @@ class Client(ClientBase):
         self,
         bundle: Bundle,
         compression: Optional[Compression] = None,
+        make_debuggable: Optional[bool] = None,
     ) -> AsyncIterator[InstalledArtifact]:
         async for response in self._install_to_destination(
-            bundle=bundle, destination=InstallRequest.APP, compression=compression
+            bundle=bundle,
+            destination=InstallRequest.APP,
+            compression=compression,
+            make_debuggable=make_debuggable,
+            bundle_id=None,
         ):
             yield response
 
     @log_and_handle_exceptions
     async def install_xctest(self, xctest: Bundle) -> AsyncIterator[InstalledArtifact]:
         async for response in self._install_to_destination(
-            bundle=xctest, destination=InstallRequest.XCTEST
+            bundle=xctest,
+            destination=InstallRequest.XCTEST,
+            compression=None,
+            make_debuggable=None,
+            bundle_id=None,
         ):
             yield response
 
     @log_and_handle_exceptions
     async def install_dylib(self, dylib: Bundle) -> AsyncIterator[InstalledArtifact]:
         async for response in self._install_to_destination(
-            bundle=dylib, destination=InstallRequest.DYLIB
+            bundle=dylib,
+            destination=InstallRequest.DYLIB,
+            compression=None,
+            make_debuggable=None,
+            bundle_id=None,
         ):
             yield response
 
     @log_and_handle_exceptions
-    async def install_dsym(self, dsym: Bundle) -> AsyncIterator[InstalledArtifact]:
+    async def install_dsym(
+        self,
+        dsym: Bundle,
+        bundle_id: Optional[str],
+        compression: Optional[Compression],
+    ) -> AsyncIterator[InstalledArtifact]:
         async for response in self._install_to_destination(
-            bundle=dsym, destination=InstallRequest.DSYM
+            bundle=dsym,
+            destination=InstallRequest.DSYM,
+            compression=compression,
+            make_debuggable=None,
+            bundle_id=bundle_id,
         ):
             yield response
 
@@ -612,13 +678,21 @@ class Client(ClientBase):
         self, framework_path: Bundle
     ) -> AsyncIterator[InstalledArtifact]:
         async for response in self._install_to_destination(
-            bundle=framework_path, destination=InstallRequest.FRAMEWORK
+            bundle=framework_path,
+            destination=InstallRequest.FRAMEWORK,
+            compression=None,
+            make_debuggable=None,
+            bundle_id=None,
         ):
             yield response
 
     @log_and_handle_exceptions
     async def push(
-        self, src_paths: List[str], container: FileContainer, dest_path: str
+        self,
+        src_paths: List[str],
+        container: FileContainer,
+        dest_path: str,
+        compression: Optional[Compression],
     ) -> None:
         async with self.stub.push.open() as stream:
             await stream.send_message(
@@ -636,10 +710,21 @@ class Client(ClientBase):
                 await stream.end()
                 await stream.recv_message()
             else:
+                if compression is not None:
+                    await stream.send_message(
+                        PushRequest(
+                            payload=Payload(compression=COMPRESSION_MAP[compression])
+                        )
+                    )
+
                 await drain_to_stream(
                     stream=stream,
                     generator=stream_map(
-                        generate_tar(paths=src_paths, verbose=self._is_verbose),
+                        generate_tar(
+                            paths=src_paths,
+                            compression=compression or Compression.GZIP,
+                            verbose=self._is_verbose,
+                        ),
                         lambda chunk: PushRequest(payload=Payload(data=chunk)),
                     ),
                     logger=self.logger,
@@ -665,6 +750,22 @@ class Client(ClientBase):
                 await drain_untar(generate_bytes(stream), output_path=dest_path)
             self.logger.info(f"pulled file to {dest_path}")
 
+    @log_and_handle_exceptions
+    async def tail(
+        self, stop: asyncio.Event, container: FileContainer, path: str
+    ) -> AsyncIterator[bytes]:
+        async with self.stub.tail.open() as stream:
+            await stream.send_message(
+                TailRequest(
+                    start=TailRequest.Start(
+                        container=file_container_to_grpc(container), path=path
+                    )
+                )
+            )
+            async for response in cancel_wrapper(stream=stream, stop=stop):
+                yield response.data
+            await stream.send_message(TailRequest(stop=TailRequest.Stop()))
+
     @log_and_handle_exceptions
     async def list_test_bundle(self, test_bundle_id: str, app_path: str) -> List[str]:
         response = await self.stub.xctest_list_tests(
@@ -859,6 +960,7 @@ class Client(ClientBase):
         foreground_if_running: bool = False,
         wait_for_debugger: bool = False,
         stop: Optional[asyncio.Event] = None,
+        pid_file: Optional[str] = None,
     ) -> None:
         async with self.stub.launch.open() as stream:
             request = LaunchRequest(
@@ -874,11 +976,12 @@ class Client(ClientBase):
             await stream.send_message(request)
             if stop:
                 await asyncio.gather(
-                    drain_launch_stream(stream), end_launch_stream(stream, stop)
+                    drain_launch_stream(stream, pid_file),
+                    end_launch_stream(stream, stop),
                 )
             else:
                 await stream.end()
-                await drain_launch_stream(stream)
+                await drain_launch_stream(stream, pid_file)
 
     @log_and_handle_exceptions
     async def record_video(self, stop: asyncio.Event, output_file: str) -> None:
@@ -963,6 +1066,38 @@ class Client(ClientBase):
                 )
                 await stream.end()
 
+    async def _handle_code_coverage_in_response(
+        self,
+        response: XctestRunResponse,
+        coverage_output_path: Optional[str],
+        coverage_format: CodeCoverageFormat,
+    ) -> None:
+        if (
+            response.code_coverage_data
+            and response.code_coverage_data.data
+            and response.code_coverage_data.data.count
+            and coverage_output_path
+        ):
+            output_path: str = coverage_output_path
+            self.logger.info(f"Decompressing code coverage to {output_path}")
+            if coverage_format == CodeCoverageFormat.EXPORTED:
+                await gunzip(
+                    response.code_coverage_data.data,
+                    output_path=output_path,
+                )
+            elif coverage_format == CodeCoverageFormat.RAW:
+                await untar_into_path(
+                    payload=response.code_coverage_data,
+                    description="raw code coverage directory",
+                    output_path=output_path,
+                    logger=self.logger,
+                )
+            self.logger.info(f"Finished decompression to {output_path}")
+        elif response.coverage_json and coverage_output_path:
+            # handle deprecated response field
+            with open(coverage_output_path, "w") as f:
+                f.write(response.coverage_json)
+
     @log_and_handle_exceptions
     async def run_xctest(
         self,
@@ -983,6 +1118,7 @@ class Client(ClientBase):
         report_attachments: bool = False,
         activities_output_path: Optional[str] = None,
         coverage_output_path: Optional[str] = None,
+        coverage_format: CodeCoverageFormat = CodeCoverageFormat.EXPORTED,
         log_directory_path: Optional[str] = None,
         wait_for_debugger: bool = False,
     ) -> AsyncIterator[TestRunInfo]:
@@ -1006,6 +1142,7 @@ class Client(ClientBase):
                 ),
                 report_attachments=report_attachments,
                 collect_coverage=coverage_output_path is not None,
+                coverage_format=coverage_format,
                 collect_logs=log_directory_path is not None,
                 wait_for_debugger=wait_for_debugger,
             )
@@ -1039,13 +1176,17 @@ class Client(ClientBase):
                         output_path=log_directory_path,
                         logger=self.logger,
                     )
-                if response.coverage_json and coverage_output_path:
-                    with open(coverage_output_path, "w") as f:
-                        f.write(response.coverage_json)
+
+                await self._handle_code_coverage_in_response(
+                    response, coverage_output_path, coverage_format
+                )
 
                 if wait_for_debugger and response.debugger.pid:
-                    print("Tests waiting for debugger. To debug run:")
-                    print(f"lldb -p {response.debugger.pid}")
+                    sys.stdout.buffer.write(
+                        json_format_debugger_info(response.debugger).encode()
+                    )
+                    sys.stdout.buffer.write(os.linesep.encode())
+                    sys.stdout.buffer.flush()
 
                 for result in make_results(response, log_parser):
                     if activities_output_path:
@@ -1089,11 +1230,34 @@ class Client(ClientBase):
             )
         )
 
+    @log_and_handle_exceptions
+    async def set_preference(
+        self, name: str, value: str, value_type: str, domain: Optional[str]
+    ) -> None:
+        await self.stub.setting(
+            SettingRequest(
+                stringSetting=SettingRequest.StringSetting(
+                    setting=AnySetting,
+                    value=value,
+                    name=name,
+                    value_type=value_type,
+                    domain=domain,
+                )
+            )
+        )
+
     @log_and_handle_exceptions
     async def get_locale(self) -> str:
         response = await self.stub.get_setting(GetSettingRequest(setting=LocaleSetting))
         return response.value
 
+    @log_and_handle_exceptions
+    async def get_preference(self, name: str, domain: Optional[str]) -> str:
+        response = await self.stub.get_setting(
+            GetSettingRequest(setting=AnySetting, name=name, domain=domain)
+        )
+        return response.value
+
     @log_and_handle_exceptions
     async def list_locale_identifiers(self) -> List[str]:
         response = await self.stub.list_settings(
@@ -1203,3 +1367,39 @@ class Client(ClientBase):
                         self.logger.info(f"Trace written to {trace_file}")
 
             return result
+
+    @log_and_handle_exceptions
+    async def dap(
+        self,
+        dap_path: str,
+        input_stream: StreamReader,
+        output_stream: StreamWriter,
+        stop: asyncio.Event,
+        compression: Optional[Compression],
+    ) -> None:
+        path = Path(dap_path)
+        pkg_id = path.stem
+        self.logger.debug("Creating dap subfolder for different pkg of dap server.")
+        await self.mkdir(container=FileContainerType.ROOT, path="dap")
+
+        ls_response = await self.ls(container=FileContainerType.ROOT, paths=["dap"])
+        installed_daps = [entry.path for entry in ls_response[0].entries]
+        if pkg_id in installed_daps:
+            self.logger.info(f"Dap pkg already exist. Id: f{pkg_id}")
+        else:
+            self.logger.info(f"Pushing {path.absolute()} to simulator dap subfolder.")
+            await self.push(
+                src_paths=[str(path.absolute())],
+                container=FileContainerType.ROOT,
+                dest_path="dap",
+                compression=compression,
+            )
+
+        async with RemoteDapServer.start(self.stub, self.logger, pkg_id) as dap_server:
+            self.logger.debug("Dap server started. Waiting for input from stdin...")
+
+            await dap_server.pipe(
+                input_stream=input_stream,
+                output_stream=output_stream,
+                stop=stop,
+            )
